<?php

namespace Drupal\commerce_payme\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\commerce_payme\HandlePaymentReturn;
use Drupal\Core\Logger\LoggerChannelFactory;

/**
 * Returns responses for Commerce Pay Me routes.
 */
class CommercePaymeReturnController extends ControllerBase {

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The request stack.
   *
   * @var \Drupal\commerce_payme\HandlePaymentReturn
   */
  protected $handlePaymentReturn;

  /**
   * Logget service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The controller constructor.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\commerce_payme\HandlePaymentReturn $handle_payment_return
   *   The handle payment return service.
   */
  public function __construct(RequestStack $request_stack, HandlePaymentReturn $handle_payment_return, LoggerChannelFactory $logger_factory) {
    $this->requestStack = $request_stack;
    $this->handlePaymentReturn = $handle_payment_return;
    $this->logger = $logger_factory->get('commerce_payme');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack'),
      $container->get('commerce_payme.handle_return'),
      $container->get('logger.factory')
    );
  }

  /**
   * Builds the response.
   */
  public function build() {
    $request = $this->requestStack->getCurrentRequest();
    $params = $request->request->all();
    $this->logger->info('RESPONSE PARAMS: ' . json_encode($params));
    // Order id.
    $order_id = $request->request->get('reserved2');
    $return_code = $this->handlePaymentReturn->handle($order_id, $params);
    $url = "/checkout/${order_id}/payment/return?code=" . $return_code;
    $response = new RedirectResponse($url);
    return $response->send();
  }

}
