<?php

namespace Drupal\commerce_payme\PluginForm;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * PaymentOffsiteForm class.
 */
class PaymeForm extends BasePaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);
    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    $operation_number = $payment->getOrderId();
    $remaining_chars = 9 - \strlen($operation_number);
    $random_portion = mt_rand(0, str_pad('9', $remaining_chars - 1, '9', STR_PAD_RIGHT));
    $operation_number .= $random_portion;
    $operation_number = str_pad($operation_number, 9, '0', STR_PAD_LEFT);
    $order = $payment->getOrder();
    $billing_profile = $order->getBillingProfile();
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $type = 'auth';
    if (!empty($form['#capture'])) {
      $type = 'sale';
    }
    $amount = $payment->getAmount();
    $amount_number = number_format(round($amount->getNumber(), 2), 2, '', '');
    $amount_number = ltrim($amount_number, '0');

    $form['acquirerId'] = [
      '#type' => 'hidden',
      '#value' => $payment_gateway_plugin->getAcquirerId(),
      '#name' => 'acquirerId',
    ];

    $form['idCommerce'] = [
      '#type' => 'hidden',
      '#value' => $payment_gateway_plugin->getCommerceId(),
      '#name' => 'idCommerce',
    ];

    $form['purchaseOperationNumber'] = [
      '#type' => 'hidden',
      '#value' => $operation_number,
      '#name' => 'purchaseOperationNumber',
    ];

    $form['purchaseAmount'] = [
      '#type' => 'hidden',
      '#value' => $amount_number,
      '#name' => 'purchaseAmount',
    ];

    $form['purchaseCurrencyCode'] = [
      '#type' => 'hidden',
      '#value' => $payment_gateway_plugin->getCurrencyCode(),
      '#name' => 'purchaseCurrencyCode',
    ];

    $form['language'] = [
      '#type' => 'hidden',
      '#value' => $payment_gateway_plugin->getLanguage(),
      '#name' => 'language',
    ];

    $form['shippingFirstName'] = [
      '#type' => 'hidden',
      '#value' => $billing_profile->address->given_name,
      '#name' => 'shippingFirstName',
    ];

    $form['shippingLastName'] = [
      '#type' => 'hidden',
      '#value' => $billing_profile->address->family_name,
      '#name' => 'shippingLastName',
    ];

    $form['shippingEmail'] = [
      '#type' => 'hidden',
      '#value' => $order->getEmail(),
      '#name' => 'shippingEmail',
    ];

    $form['shippingAddress'] = [
      '#type' => 'hidden',
      '#value' => $billing_profile->address->address_line1,
      '#name' => 'shippingAddress',
    ];

    $form['shippingZIP'] = [
      '#type' => 'hidden',
      '#value' => $billing_profile->address->postal_code,
      '#name' => 'shippingZIP',
    ];

    $form['shippingCity'] = [
      '#type' => 'hidden',
      '#value' => $billing_profile->address->locality,
      '#name' => 'shippingCity',
    ];

    $form['shippingState'] = [
      '#type' => 'hidden',
      '#value' => $billing_profile->address->administrative_area,
      '#name' => 'shippingState',
    ];

    $form['shippingCountry'] = [
      '#type' => 'hidden',
      '#value' => $billing_profile->address->country_code,
      '#name' => 'shippingCountry',
    ];

    $form['descriptionProducts'] = [
      '#type' => 'hidden',
      '#value' => 'Order: ' . $order->id(),
      '#name' => 'descriptionProducts',
    ];

    $form['programmingLanguage'] = [
      '#type' => 'hidden',
      '#value' => 'PHP',
      '#name' => 'programmingLanguage',
    ];

    $form['purchaseVerification'] = [
      '#type' => 'hidden',
      '#value' => $payment_gateway_plugin->getHash([
        $payment_gateway_plugin->getAcquirerId(),
        $payment_gateway_plugin->getCommerceId(),
        $operation_number,
        $amount_number,
        $payment_gateway_plugin->getCurrencyCode(),
        $payment_gateway_plugin->getKey(),
      ]),
      '#name' => 'purchaseVerification',
    ];

    if ($payment_gateway_plugin->getMode() === 'test') {
      $on_click = 'javascript:AlignetVPOS2.openModal("' . $payment_gateway_plugin->getUrl() . '/")';
    }
    else {
      // @TODO: Design id should be configurable.
      $on_click = 'javascript:AlignetVPOS2.openModal("", "2")';
    }
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => t('Buy'),
      '#attributes' => [
        'onclick' => $on_click,
      ],
    ];

    $form['reserved2'] = [
      '#type' => 'hidden',
      '#value' => $payment->getOrderId(),
      '#name' => 'reserved2',
    ];

    $form['environment'] = [
      '#type' => 'value',
      '#value' => $payment_gateway_plugin->getMode(),
    ];

    $form['submit']['#process'] = [[get_class($this), 'processForm']];

    return $form;
  }

  /**
   * Process callback for form.
   */
  public static function processForm(array $element, FormStateInterface $form_state, &$complete_form) {
    $complete_form['#action'] = '#';
    if ($complete_form['payment_process']['offsite_payment']['environment']['#value'] === 'live') {
      $complete_form['#attached']['library'][] = 'commerce_payme/commerce_modal_prod';
    }
    else {
      $complete_form['#attached']['library'][] = 'commerce_payme/commerce_modal_test';
    }
    $complete_form['#attributes']['class'][] = 'alignet-form-vpos2';
    return $element;
  }

}
